
-- SurgeonGeneral by Brimmstone of Area 52 (US)
-- Version 1.0
--	Initial Version

SurgeonGeneral = LibStub("AceAddon-3.0"):NewAddon("SurgeonGeneral", "AceConsole-3.0", "AceEvent-3.0")

local options = {
	name = "SurgeonGeneral",
	handler = SurgeonGeneral,
    type = "group",
    args = {
		Reset = {
			type = "execute",
			name = "Reset",
			desc = "Reset Window Position",
			func = "ResetWindowPosition",
		},
		Verbose = {
			type = "toggle",
			name = "Verbose",
			desc = "Turns on Verbose Mode",
			set = "ToggleVerbose",
			get = "GetVerbose",
		},
	},
}

function SurgeonGeneral:OnInitialize()
	LibStub("AceConfig-3.0"):RegisterOptionsTable("SurgeonGeneral", options);
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("SurgeonGeneral", "SurgeonGeneral");
    self:RegisterChatCommand("sg", "ChatCommand");
	
	hooksecurefunc("NotifyInspect", SurgeonGeneral_NotifyInspect);
end

function SurgeonGeneral:ChatCommand(input)
	local x
	if not input or input:trim() == "" then
		SurgeonGeneralConfigFrame:Show();
	else
		LibStub("AceConfigCmd-3.0").HandleCommand(SurgeonGeneral, "sg", "SurgeonGeneral", input)
	end
end

function SurgeonGeneral:ResetWindowPosition()
	SurgeonGeneral:ClearAllPoints();
	SurgeonGeneral:SetPoint("CENTER","UIParent");
end

function SurgeonGeneral:ToggleVerbose()
	options.Verbose = not options.Verbose
end

function SurgeonGeneral:GetVerbose()
	return options.Verbose;
end

function CopyBossFromTarget()
	local name;
	if (UnitExists("target")) then
		if (UnitIsPlayer("target")) then
			return;
		else
			name,_ = UnitName("target");
			getglobal("BossName"):SetText(name);
		end
	end
end

function SurgeonGeneral:OnLoad()
	getglobal("Phase1Name"):SetText("Phase1");
	getglobal("Phase2Name"):SetText("Phase2");
	getglobal("Phase3Name"):SetText("Phase3");
	getglobal("Phase4Name"):SetText("Phase4");
	getglobal("Phase5Name"):SetText("Phase5");
	SurgeonGeneral:RegisterEvent("INSPECT_TALENT_READY","AutoFillInspectReady");
end

function SetVar()
	--SurgeonGeneral:Print("Set Var");
end

function CalcPhases()
	local NumberOfPhases = 1;
	local k;
	k = 2;
	while k < 6 do
		if getglobal("Phase"..k.."Enable"):GetChecked() then
			NumberOfPhases = k;
			k = k + 1;
		else
			k = 6;
		end
	end
	if NumberOfPhases < 4 then
		Phase5Enable:Disable();
	else
		Phase5Enable:Enable();
	end
	
	if NumberOfPhases < 3 then
		Phase4Enable:Disable();
	else
		Phase4Enable:Enable();
	end
	
	if NumberOfPhases < 2 then
		Phase3Enable:Disable();
	else
		Phase3Enable:Enable();
	end
	
	return NumberOfPhases;
end

function PhaseNameChanged()
	-- placeholder
end

function ValidatePhaseNames()
	if (getglobal("Phase1Name"):GetText() == "") then
		getglobal("Phase1Name"):SetText("Phase1");
	end
	if (getglobal("Phase2Name"):GetText() == "") then
		getglobal("Phase2Name"):SetText("Phase2");
	end
	if (getglobal("Phase3Name"):GetText() == "") then
		getglobal("Phase3Name"):SetText("Phase3");
	end
	if (getglobal("Phase4Name"):GetText() == "") then
		getglobal("Phase4Name"):SetText("Phase4");
	end
	if (getglobal("Phase5Name"):GetText() == "") then
		getglobal("Phase5Name"):SetText("Phase5");
	end
end

function PhaseEnable_OnClick()
	CalcPhases();
end

function ReportButton_OnClick()
	if getglobal("RaidChannelEnable"):GetChecked() then
		AnnounceAssignments("RAID");
	end
	
	if getglobal("CustomChannelEnable"):GetChecked() then
		AnnounceAssignments(getglobal("CustomChannelName"):GetText());
	end
end
	
function AnnounceAssignments(ReportChannel)
	local HealerMessage;
	local HealerClass;
	local j,k;
	local ReportChannelNumber;
	local ReportChannelName;
	
	ValidatePhaseNames();
	
	if (ReportChannel == "RAID") then
		ReportChannelNumber = nil;
		ReportChannelName = "RAID"
	else	
		ReportChannelNumber,_ = GetChannelName(ReportChannel);
		ReportChannelName = "CHANNEL"
		if(ReportChannelNumber == 0) then
			return;
		end
	end
	
	--SendChatMessage(" ",ReportChannelName,nil,ReportChannelNumber);
	if not (getglobal("BossName"):GetText() == "") then
		SendChatMessage("Healing Assignments: "..getglobal("BossName"):GetText(),ReportChannelName,nil,ReportChannelNumber);
	else
		SendChatMessage("Healing Assignments",ReportChannelName,nil,ReportChannelNumber);
	end
	
	HealerMessage = "Healer:  ";
	HealerMessage = HealerMessage .. getglobal("Phase1Name"):GetText();
	for j = 2,CalcPhases() do
		HealerMessage = HealerMessage .. " / " .. getglobal("Phase"..j.."Name"):GetText();
	end
	SendChatMessage(HealerMessage,ReportChannelName,nil,ReportChannelNumber);
	
	for k = 1,10 do
		if not (getglobal("HealerName"..k):GetText() == "") then
			HealerMessage = getglobal("HealerName"..k):GetText() .. ":  ";
			HealerMessage = HealerMessage .. getglobal("Healer"..k.."Phase1"):GetText();
			for j = 2,CalcPhases() do
				HealerMessage = HealerMessage .. " / " .. getglobal("Healer"..k.."Phase"..j):GetText();
			end
			SendChatMessage(HealerMessage,ReportChannelName,nil,ReportChannelNumber);
		end
	end
	--SendChatMessage(" ",ReportChannelName,nil,ReportChannelNumber);
end

local AutoFillInProgress
local AutoFillCurrentUnit
local AutoFillCurrentUnitClassE
local AutoFillCurrentUnitClassL
local AutoFillCurrentID
local AutoFillTaint
local AutoFillCurrentSlot

function VerbosePrint(msg,color)
	if (options.Verbose) then
		if color ~= nil then
			DEFAULT_CHAT_FRAME:AddMessage("|cff33ff99Surgeon General: |cff"..color..msg.."|r");
		else
			DEFAULT_CHAT_FRAME:AddMessage("|cff33ff99Surgeon General: |cffffffff"..msg.."|r");
		end
	end
end

function AutoFillStart()
	if (not UnitInRaid("player")) then
		return;
	end

	AutoFillInProgress = 1;
	AutoFillCurrentID = 0;
	AutoFillCurrentSlot = 1;
	
	AutoFillNext();	
end

function AutoFillNext()
	if (AutoFillInProgress == 1) then
		while (AutoFillCurrentID < GetNumRaidMembers()) do
			AutoFillCurrentID = AutoFillCurrentID + 1;
			AutoFillCurrentUnit = "raid"..AutoFillCurrentID;
			AutoFillCurrentUnitClassL,AutoFillCurrentUnitClassE = UnitClass(AutoFillCurrentUnit);			
			if ((AutoFillCurrentUnitClassE == "WARRIOR") or (AutoFillCurrentUnitClassE == "ROGUE") or (AutoFillCurrentUnitClassE == "HUNTER") or (AutoFillCurrentUnitClassE == "MAGE") or (AutoFillCurrentUnitClassE == "WARLOCK")) then
				-- If it's not a healing class, go to next unit
				VerbosePrint(AutoFillCurrentID..") "..UnitName(AutoFillCurrentUnit).." is not a healing class.");
			elseif UnitIsUnit("player",AutoFillCurrentUnit) then
				-- If it's me, just call the function to get my info (You can't inspect yourself).
				VerbosePrint(AutoFillCurrentID..") "..UnitName(AutoFillCurrentUnit).." is me.");
				AutoFillInspectReady();
			elseif (CheckInteractDistance(AutoFillCurrentUnit,1)) then
				-- If it's somebody else and they're within inspect range, get their info.
				VerbosePrint(AutoFillCurrentID..") ".."Inspecting "..UnitName(AutoFillCurrentUnit));
				NotifyInspect(AutoFillCurrentUnit);
				return;
			else
				-- If they're not within range, just skip them and move along
				VerbosePrint(AutoFillCurrentID..") "..UnitName(AutoFillCurrentUnit).." was out of inspection range.");
			end
		end
		AutoFillInProgress = 0;
	end
end

function AutoFillRepeat()
	if (AutoFillInProgress == 1) then
		NotifyInspect(AutoFillCurrentUnit);
	end
end

function AutoFillStop()
	AutoFillInProgress = 0;
end

function SurgeonGeneral_NotifyInspect(unit)
	if (AutoFillInProgress == 0) then
		return;
	elseif (unit == AutoFillCurrentUnit) then
		AutoFillTaint = 0;
	else
		AutoFillTaint = 1;
		VerbosePrint("Inspect Taint!","ff0000");
		AutoFillRepeat();
	end
end

function SurgeonGeneral:AutoFillInspectReady()
	AutoFillInspectReady();
end

function AutoFillInspectReady()
	local HealingTalents;
	local HealingTree;
	local HealingTreeName;
	
	if (AutoFillInProgress == 0) then
		return;
	elseif (AutoFillTaint == 1) then
		return;
	end
	
	VerbosePrint(AutoFillCurrentID..") "..UnitName(AutoFillCurrentUnit).." is a "..AutoFillCurrentUnitClassL);
	
	-- Set appropriate healing tree ID for class
	if (AutoFillCurrentUnitClassE == "PALADIN") then
		HealingTree = 1;
	elseif (AutoFillCurrentUnitClassE == "PRIEST") then
		HealingTree = 2;
	elseif (AutoFillCurrentUnitClassE == "SHAMAN") then
		HealingTree = 3;
	elseif (AutoFillCurrentUnitClassE == "DRUID") then
		HealingTree = 3;
	else
		--  Not a healing class... should not have been inspected, but trapping anyways.
		return;
	end
	
	if (UnitIsUnit("player",AutoFillCurrentUnit)) then
		HealingTreeName,_,HealingTalents,_ = GetTalentTabInfo(HealingTree,false);
	else
		HealingTreeName,_,HealingTalents,_ = GetTalentTabInfo(HealingTree,true);
	end
	
	-- Assuming if you call yourself a healer, you've put at least 36 talents in your healing tree.
	VerbosePrint(AutoFillCurrentID..") "..UnitName(AutoFillCurrentUnit).." has "..HealingTalents.." in the "..HealingTreeName.." tree.");
	if (HealingTalents > 35) then
		local HealerName,_ = UnitName(AutoFillCurrentUnit);
		getglobal("HealerName"..AutoFillCurrentSlot):SetText(HealerName);
		AutoFillCurrentSlot = AutoFillCurrentSlot + 1;
	end
	
	if AutoFillCurrentSlot < 11 then
		AutoFillNext();
	else
		AutoFillStop();
	end
end


function RaidChannelEnable_OnClick()

end

function CustomChannelEnable_OnClick()

end

function EditBox_OnEscapePressed()
	SurgeonGeneralConfigFrame:Hide()
end
